package ctbrec.ui.sites.streamate;

import static ctbrec.io.HttpConstants.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import ctbrec.Config;
import ctbrec.Model;
import ctbrec.io.HttpException;
import ctbrec.sites.streamate.Streamate;
import ctbrec.sites.streamate.StreamateHttpClient;
import ctbrec.sites.streamate.StreamateModel;
import ctbrec.ui.PaginatedScheduledService;
import javafx.concurrent.Task;
import okhttp3.Request;
import okhttp3.Response;

public class StreamateFollowedService extends PaginatedScheduledService {

    private static final Logger LOG = LoggerFactory.getLogger(StreamateFollowedService.class);

    private static final int MODELS_PER_PAGE = 16;
    private Streamate streamate;
    private StreamateHttpClient httpClient;
    private String url;
    private boolean showOnline = true;

    public StreamateFollowedService(Streamate streamate) {
        this.streamate = streamate;
        this.httpClient = (StreamateHttpClient) streamate.getHttpClient();
        this.url = "https://member.naiadsystems.com/search/favorites?domain=streamate.com&skipXmentSelection=true";
    }

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<List<Model>>() {
            @Override
            public List<Model> call() throws IOException, SAXException, ParserConfigurationException, XPathExpressionException {
                httpClient.login();
                String saKey = httpClient.getSaKey();
                String pageUrl = url + "&from=" + ((page - 1) * MODELS_PER_PAGE) + "&size=" + MODELS_PER_PAGE;
                LOG.debug("Fetching page {}", pageUrl);
                Request request = new Request.Builder()
                        .url(pageUrl)
                        .header(USER_AGENT, Config.getInstance().getSettings().httpUserAgent)
                        .header(ACCEPT, MIMETYPE_APPLICATION_JSON)
                        .header(ACCEPT_LANGUAGE, Locale.ENGLISH.getLanguage())
                        .header(ORIGIN, streamate.getBaseUrl())
                        .header(REFERER, streamate.getBaseUrl() + "/view/favorites")
                        .header("sakey", saKey)
                        .header("platform", "SCP")
                        .header("smtid", UUID.randomUUID().toString() + "G0211569057409")
                        .header("smeid", UUID.randomUUID().toString() + "G0211569057409")
                        .header("smvid", UUID.randomUUID().toString() + "G0211569057409")
                        .build();
                try(Response response = streamate.getHttpClient().execute(request)) {
                    if (response.isSuccessful()) {
                        List<Model> models = new ArrayList<>();
                        String content = response.body().string();
                        JSONObject json = new JSONObject(content);
                        if (json.has("totalResultCount")) {
                            JSONArray performers = json.getJSONArray("performers");
                            for (int i = 0; i < performers.length(); i++) {
                                JSONObject p = performers.getJSONObject(i);
                                String nickname = p.getString("nickname");
                                StreamateModel model = (StreamateModel) streamate.createModel(nickname);
                                model.setId(p.getLong("id"));
                                model.setPreview(p.getString("thumbnail"));
                                boolean online = p.optBoolean("online") && notPrivateEtc(p);
                                model.setOnline(online);
                                if (online == showOnline) {
                                    models.add(model);
                                }
                            }
                        } else {
                            throw new IOException("Response: " + json.optString("message"));
                        }
                        return models;
                    } else {
                        throw new HttpException(response.code(), response.message());
                    }
                }
            }

            private boolean notPrivateEtc(JSONObject p) {
                if (p.has("liveState")) {
                    JSONObject liveState = p.getJSONObject("liveState");
                    boolean offline = liveState.optBoolean("onBreak")
                            || liveState.optBoolean("privateChat")
                            || liveState.optBoolean("exclusiveShow")
                            || liveState.optBoolean("specialShow")
                            || liveState.optBoolean("goldShow");
                    return !offline;
                }
                return false;
            }
        };
    }

    public void setOnline(boolean online) {
        this.showOnline = online;
    }
}
